@@ -3,6 +3,7 @@
  *	ppc_opc.cc
  *
  *	Copyright (C) 2003, 2004 Sebastian Biallas (sb@biallas.net)
+ *	Copyright (C) 2004 Daniel Foesch (dfoesch@cs.nmsu.edu)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -59,7 +60,7 @@ static void FASTCALL writeDEC(uint32 newdec)
 		// as a result of a memory overwrite or something else. Let's handle
 		// that until we figure out why.
 		if (q > 20 * 1000 * 1000) {
-			PPC_OPC_WARN("write dec > 20 millisec := %ld (%qu)\n", gCPU.dec, q);
+			PPC_OPC_WARN("write dec > 20 millisec := %08x (%qu)\n", gCPU.dec, q);
 			q = 10 * 1000 * 1000;
 			sys_set_timer(gDECtimer, 0, q, false);
 		} else {
@@ -70,6 +71,18 @@ static void FASTCALL writeDEC(uint32 newdec)
 	gDECwriteITB = ppc_get_cpu_ideal_timebase();
 }
 
+static void FASTCALL writeTBL(uint32 newtbl)
+{
+	uint64 tbBase = ppc_get_cpu_timebase();
+	gCPU.tb = (tbBase & 0xffffffff00000000ULL) | (uint64)newtbl;
+}
+									
+static void FASTCALL writeTBU(uint32 newtbu)
+{
+	uint64 tbBase = ppc_get_cpu_timebase();
+	gCPU.tb = ((uint64)newtbu << 32) | (tbBase & 0xffffffff);
+}
+
 void ppc_set_msr(uint32 newmsr)
 {
 /*	if ((newmsr & MSR_EE) && !(gCPU.msr & MSR_EE)) {
@@ -103,6 +116,7 @@ void ppc_opc_gen_check_privilege()
 	if (!gJITC.checkedPriviledge) {
 		jitcClobberCarryAndFlags();
 		jitcFloatRegisterClobberAll();
+		jitcFlushVectorRegister();
 		NativeReg msr = jitcGetClientRegisterMapping(PPC_MSR);
 		if (msr == REG_NO) {
 			asmTESTDMemImm((uint32)&gCPU.msr, MSR_PR);
@@ -125,17 +139,16 @@ static inline void ppc_opc_gen_set_pc_rel(uint32 li)
 	li += gJITC.pc;
 	if (li < 4096) {
 		/*
-		 *	No need to set ESI
+		 *	We assure here 7+6+5+5 bytes, to have enough space for 
+		 *	four instructions (since we want to modify them)
 		 */
-		asmALURegImm(X86_MOV, EAX, li);
+		jitcEmitAssure(7+6+5+5);
+		
+		asmMOVRegImm_NoFlags(EAX, li);
 		asmCALL((NativeAddress)ppc_heartbeat_ext_rel_asm);
-		jitcEmitAssure(10);
-		/*
-		 *	We assure here 10 bytes, to have enough space for 
-		 *	next two instructions (since we want to modify them)
-		 */
 		asmMOVRegImm_NoFlags(EAX, li);
 		asmCALL((NativeAddress)ppc_new_pc_this_page_asm);
+		asmNOP(3);
 	} else {
 		asmALURegImm(X86_MOV, EAX, li);
 		asmJMP((NativeAddress)ppc_new_pc_rel_asm);
@@ -169,7 +182,6 @@ JITCFlow ppc_opc_gen_bx()
 		asmALURegImm(X86_ADD, EAX, gJITC.pc+4);
 		asmMOVDMemReg((uint32)&gCPU.lr, EAX);
 	}
-	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	if (gJITC.current_opc & PPC_OPC_AA) {
 		asmALURegImm(X86_MOV, EAX, li);
 		asmJMP((NativeAddress)ppc_new_pc_asm);
@@ -210,6 +222,7 @@ JITCFlow ppc_opc_gen_bcx()
 	PPC_OPC_TEMPL_B(gJITC.current_opc, BO, BI, BD);
 	NativeAddress fixup = NULL;
 	jitcFloatRegisterClobberAll();
+	jitcFlushVectorRegister();
 	if (!(BO & 16)) {
 		// only branch if condition
 		if (BO & 4) {
@@ -255,7 +268,6 @@ JITCFlow ppc_opc_gen_bcx()
 					asmALURegImm(X86_ADD, EAX, gJITC.pc+4);
 					asmMOVDMemReg((uint32)&gCPU.lr, EAX);
 				}
-				asmALURegImm(X86_MOV, ESI, gJITC.pc);
 				if (gJITC.current_opc & PPC_OPC_AA) {
 					asmALURegImm(X86_MOV, EAX, BD);
 					asmJMP((NativeAddress)ppc_new_pc_asm);
@@ -287,7 +299,6 @@ JITCFlow ppc_opc_gen_bcx()
 				asmALURegImm(X86_ADD, EAX, gJITC.pc+4);
 				asmMOVDMemReg((uint32)&gCPU.lr, EAX);
 			}
-			asmALURegImm(X86_MOV, ESI, gJITC.pc);
 			if (gJITC.current_opc & PPC_OPC_AA) {
 				asmALURegImm(X86_MOV, EAX, BD);
 				asmJMP((NativeAddress)ppc_new_pc_asm);
@@ -309,7 +320,6 @@ JITCFlow ppc_opc_gen_bcx()
 				asmALURegImm(X86_ADD, EAX, gJITC.pc+4);
 				asmMOVDMemReg((uint32)&gCPU.lr, EAX);
 			}
-			asmALURegImm(X86_MOV, ESI, gJITC.pc);
 			if (gJITC.current_opc & PPC_OPC_AA) {
 				asmALURegImm(X86_MOV, EAX, BD);
 				asmJMP((NativeAddress)ppc_new_pc_asm);
@@ -331,7 +341,6 @@ JITCFlow ppc_opc_gen_bcx()
 		asmALURegImm(X86_ADD, EAX, gJITC.pc+4);
 		asmMOVDMemReg((uint32)&gCPU.lr, EAX);
 	}
-	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	if (gJITC.current_opc & PPC_OPC_AA) {
 		asmALURegImm(X86_MOV, EAX, BD);
 		asmJMP((NativeAddress)ppc_new_pc_asm);
@@ -366,6 +375,7 @@ JITCFlow ppc_opc_gen_bcctrx()
 	uint32 BO, BI, BD;
 	PPC_OPC_TEMPL_XL(gJITC.current_opc, BO, BI, BD);
 	jitcFloatRegisterClobberAll();
+	jitcFlushVectorRegister();
 	if (BO & 16) {
 		// branch always
 		jitcClobberCarryAndFlags();
@@ -376,7 +386,6 @@ JITCFlow ppc_opc_gen_bcctrx()
 			asmALURegImm(X86_ADD, ECX, gJITC.pc+4);
 			asmMOVDMemReg((uint32)&gCPU.lr, ECX);
 		}
-		asmALURegImm(X86_MOV, ESI, gJITC.pc);
 		asmALURegImm(X86_AND, EAX, 0xfffffffc);
 		asmJMP((NativeAddress)ppc_new_pc_asm);
 		return flowEndBlockUnreachable;
@@ -392,7 +401,6 @@ JITCFlow ppc_opc_gen_bcctrx()
 			asmALURegImm(X86_ADD, ECX, gJITC.pc+4);
 			asmMOVDMemReg((uint32)&gCPU.lr, ECX);
 		}
-		asmALURegImm(X86_MOV, ESI, gJITC.pc);
 		asmALURegImm(X86_AND, EAX, 0xfffffffc);
 		asmJMP((NativeAddress)ppc_new_pc_asm);
 		asmResolveFixup(fixup, asmHERE());	
@@ -431,6 +439,7 @@ JITCFlow ppc_opc_gen_bclrx()
 		PPC_OPC_ERR("not impl.: bclrx + BO&4\n");
 	}
 	jitcFloatRegisterClobberAll();
+	jitcFlushVectorRegister();
 	if (BO & 16) {
 		// branch always
 		jitcClobberCarryAndFlags();
@@ -441,7 +450,6 @@ JITCFlow ppc_opc_gen_bclrx()
 			asmALURegImm(X86_ADD, ECX, gJITC.pc+4);
 			asmMOVDMemReg((uint32)&gCPU.lr, ECX);
 		}
-		asmALURegImm(X86_MOV, ESI, gJITC.pc);
 		asmALURegImm(X86_AND, EAX, 0xfffffffc);
 		asmJMP((NativeAddress)ppc_new_pc_asm);
 		return flowEndBlockUnreachable;
@@ -457,7 +465,6 @@ JITCFlow ppc_opc_gen_bclrx()
 			asmALURegImm(X86_ADD, ECX, gJITC.pc+4);
 			asmMOVDMemReg((uint32)&gCPU.lr, ECX);
 		}
-		asmALURegImm(X86_MOV, ESI, gJITC.pc);
 		asmALURegImm(X86_AND, EAX, 0xfffffffc);
 		asmJMP((NativeAddress)ppc_new_pc_asm);
 		asmResolveFixup(fixup, asmHERE());
@@ -598,7 +605,6 @@ JITCFlow ppc_opc_gen_icbi()
 	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	asmCALL((NativeAddress)ppc_opc_icbi_asm);
 	asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 	return flowEndBlockUnreachable;
 }
@@ -760,9 +766,14 @@ JITCFlow ppc_opc_gen_mfmsr()
 	return flowContinue;
 }
 
-void FASTCALL blbl(uint32 a)
+void FASTCALL unknown_tbr_warning(uint32 a, uint32 spr1, uint32 spr2)
 {
-	PPC_OPC_ERR("invalid spr @%08x\n", a);
+	PPC_OPC_WARN("invalid tbr %d:%d  @%08x\n", spr1, spr2, a);
+}
+
+void FASTCALL unknown_spr_warning(uint32 a, uint32 spr1, uint32 spr2)
+{
+	PPC_OPC_WARN("invalid spr %d:%d  @%08x\n", spr1, spr2, a);
 }
 
 /*
@@ -780,6 +791,11 @@ void ppc_opc_mfspr()
 		case 8: gCPU.gpr[rD] = gCPU.lr; return;
 		case 9: gCPU.gpr[rD] = gCPU.ctr; return;
 		}
+	case 8:	// altivec makes this user visible
+		if (spr1 == 0) {
+			gCPU.gpr[rD] = gCPU.vrsave; 
+			return;
+		}
 	}
 	if (gCPU.msr & MSR_PR) {
 		ppc_exception(PPC_EXC_PROGRAM, PPC_EXC_PROGRAM_PRIV);
@@ -828,6 +844,36 @@ void ppc_opc_mfspr()
 		case 31: gCPU.gpr[rD] = gCPU.dbatl[3]; return;
 		}
 		break;
+	case 29:
+		switch (spr1) {
+		case 16:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 17:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 18:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 24:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 25:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 26:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 28:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 29:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 30:
+			gCPU.gpr[rD] = 0;
+			return;
+		}
 	case 31:
 		switch (spr1) {
 		case 16:
@@ -838,6 +884,12 @@ void ppc_opc_mfspr()
 			PPC_OPC_WARN("read from spr %d:%d (HID1) not supported!\n", spr1, spr2);
 			gCPU.gpr[rD] = gCPU.hid[1];
 			return;
+		case 22:
+			gCPU.gpr[rD] = 0;
+			return;
+		case 23:
+			gCPU.gpr[rD] = 0;
+			return;
 		case 25:
 			PPC_OPC_WARN("read from spr %d:%d (L2CR) not supported! (from %08x)\n", spr1, spr2, gCPU.pc);
 			gCPU.gpr[rD] = 0;
@@ -858,8 +910,13 @@ void ppc_opc_mfspr()
 //			PPC_OPC_WARN("read from spr %d:%d (THRM3) not supported!\n", spr1, spr2);
 			gCPU.gpr[rD] = 0;
 			return;
+		case 31:
+//			PPC_OPC_WARN("read from spr %d:%d (???) not supported!\n", spr1, spr2);
+			gCPU.gpr[rD] = 0;
+			return;
 		}
 	}
+	fprintf(stderr, "unknown mfspr: %i:%i\n", spr1, spr2);
 	SINGLESTEP("invalid mfspr\n");
 }
 
@@ -884,6 +941,11 @@ JITCFlow ppc_opc_gen_mfspr()
 		case 8: move_reg(PPC_GPR(rD), PPC_LR); return flowContinue;
 		case 9: move_reg(PPC_GPR(rD), PPC_CTR); return flowContinue;
 		}
+	case 8:
+		if (spr1 == 0) {
+			move_reg(PPC_GPR(rD), PPC_VRSAVE); 
+			return flowContinue;
+		}
 	}
 	ppc_opc_gen_check_privilege();
 	switch (spr2) {
@@ -944,22 +1006,41 @@ JITCFlow ppc_opc_gen_mfspr()
 		case 31: move_reg(PPC_GPR(rD), PPC_DBATL(3)); return flowContinue;
 		}
 		break;
+	case 29:
+		switch (spr1) {
+		case 16: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 17: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 18: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 24: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 25: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 26: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 28: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 29: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 30: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		}
+		break;
 	case 31:
 		switch (spr1) {
 		case 16: move_reg(PPC_GPR(rD), PPC_HID0); return flowContinue;
 		case 17: move_reg(PPC_GPR(rD), PPC_HID1); return flowContinue;
+		case 22: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
+		case 23: move_reg0(PPC_GPR(rD)); return flowContinue; //g4
 		case 25: move_reg0(PPC_GPR(rD)); return flowContinue;
 		case 27: move_reg0(PPC_GPR(rD)); return flowContinue;
 		case 28: move_reg0(PPC_GPR(rD)); return flowContinue;
 		case 29: move_reg0(PPC_GPR(rD)); return flowContinue;
 		case 30: move_reg0(PPC_GPR(rD)); return flowContinue;
+		case 31: move_reg0(PPC_GPR(rD)); return flowContinue;
 		}
 	}
+	move_reg0(PPC_GPR(rD));
+	jitcClobberAll();
 	asmMOVRegDMem(EAX, (uint32)&gCPU.current_code_base);
-	asmJMP((NativeAddress)blbl);
-	return flowContinue;
-	PPC_OPC_ERR("unknown spr %d:%d\n", spr1, spr2);
-	return flowEndBlockUnreachable;
+	asmALURegImm(X86_ADD, EAX, gJITC.pc);
+	asmALURegImm(X86_MOV, EDX, spr1);
+	asmALURegImm(X86_MOV, ECX, spr2);
+	asmCALL((NativeAddress)unknown_spr_warning);
+	return flowEndBlock;
 }
 /*
  *	mfsr		Move from Segment Register
@@ -1037,7 +1118,31 @@ void ppc_opc_mftb()
 }
 JITCFlow ppc_opc_gen_mftb()
 {
-	ppc_opc_gen_interpret(ppc_opc_mftb);
+	int rD, spr1, spr2;
+	PPC_OPC_TEMPL_X(gJITC.current_opc, rD, spr1, spr2);
+	switch (spr2) {
+	case 8:
+		switch (spr1) {
+		case 12:
+			jitcClobberAll();
+			asmCALL((NativeAddress)ppc_get_cpu_timebase);
+			jitcMapClientRegisterDirty(PPC_GPR(rD), NATIVE_REG | EAX);
+			return flowContinue;
+							
+		case 13:
+			jitcClobberAll();
+			asmCALL((NativeAddress)ppc_get_cpu_timebase);
+			jitcMapClientRegisterDirty(PPC_GPR(rD), NATIVE_REG | EDX);
+			return flowContinue;
+		}
+		break;
+	}
+	move_reg0(PPC_GPR(rD));
+	jitcClobberAll();
+	asmMOVRegDMem(EAX, (uint32)&gCPU.current_code_base);
+	asmALURegImm(X86_MOV, EDX, spr1);
+	asmALURegImm(X86_MOV, ECX, spr2);
+	asmCALL((NativeAddress)unknown_tbr_warning);
 	return flowEndBlock;
 }
 /*
@@ -1277,7 +1382,6 @@ JITCFlow ppc_opc_gen_mtmsr()
 	jitcGetClientRegister(PPC_GPR(rS), NATIVE_REG | EAX);
 	asmCALL((NativeAddress)ppc_set_msr_asm);
 	asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 //	return flowContinue;
 	return flowEndBlockUnreachable;
@@ -1300,6 +1404,11 @@ void ppc_opc_mtspr()
 		case 8:	gCPU.lr = gCPU.gpr[rS]; return;
 		case 9:	gCPU.ctr = gCPU.gpr[rS]; return;
 		}
+	case 8:
+		if (spr1 == 0) {
+			gCPU.vrsave = gCPU.gpr[rS]; 
+			return;
+		}
 	}
 	if (gCPU.msr & MSR_PR) {
 		ppc_exception(PPC_EXC_PROGRAM, PPC_EXC_PROGRAM_PRIV);
@@ -1329,9 +1438,9 @@ void ppc_opc_mtspr()
 		case 17: gCPU.sprg[1] = gCPU.gpr[rS]; return;
 		case 18: gCPU.sprg[2] = gCPU.gpr[rS]; return;
 		case 19: gCPU.sprg[3] = gCPU.gpr[rS]; return;
+		case 28: writeTBL(gCPU.gpr[rS]); return;
+		case 29: writeTBU(gCPU.gpr[rS]); return;
 /*		case 26: gCPU.gpr[rD] = gCPU.ear; return;
-		case 28: TB (lower)
-		case 29: TB (upper)
 		case 31: gCPU.gpr[rD] = gCPU.pvr; return;*/
 		}
 		break;
@@ -1395,18 +1504,35 @@ void ppc_opc_mtspr()
 			return;
 		}
 		break;
+	case 29:
+		switch(spr1) {
+		case 17: return;
+		case 24: return;
+		case 25: return;
+		case 26: return;
+		}
 	case 31:
 		switch (spr1) {
 		case 16:
 //			PPC_OPC_WARN("write(%08x) to spr %d:%d (HID0) not supported! @%08x\n", gCPU.gpr[rS], spr1, spr2, gCPU.pc);
 			gCPU.hid[0] = gCPU.gpr[rS];
 			return;
+		case 17:
+			PPC_OPC_WARN("write(%08x) to spr %d:%d (HID1) not supported! @%08x\n", gCPU.gpr[rS], spr1, spr2, gCPU.pc);
+			gCPU.hid[1] = gCPU.gpr[rS];
+			return;
 		case 18:
 			PPC_OPC_ERR("write(%08x) to spr %d:%d (IABR) not supported!\n", gCPU.gpr[rS], spr1, spr2);
 			return;
 		case 21:
 			PPC_OPC_ERR("write(%08x) to spr %d:%d (DABR) not supported!\n", gCPU.gpr[rS], spr1, spr2);
 			return;
+		case 22:
+			PPC_OPC_ERR("write(%08x) to spr %d:%d (?) not supported!\n", gCPU.gpr[rS], spr1, spr2);
+			return;
+		case 23:
+			PPC_OPC_ERR("write(%08x) to spr %d:%d (?) not supported!\n", gCPU.gpr[rS], spr1, spr2);
+			return;
 		case 27:
 			PPC_OPC_WARN("write(%08x) to spr %d:%d (ICTC) not supported!\n", gCPU.gpr[rS], spr1, spr2);
 			return;
@@ -1419,8 +1545,11 @@ void ppc_opc_mtspr()
 		case 30:
 //			PPC_OPC_WARN("write(%08x) to spr %d:%d (THRM3) not supported!\n", gCPU.gpr[rS], spr1, spr2);
 			return;
+		case 31: 
+			return;
 		}
 	}
+	fprintf(stderr, "unknown mtspr: %i:%i\n", spr1, spr2);
 	SINGLESTEP("unknown mtspr\n");
 }
 
@@ -1463,6 +1592,11 @@ JITCFlow ppc_opc_gen_mtspr()
 		case 8:	move_reg(PPC_LR, PPC_GPR(rS)); return flowContinue;
 		case 9:	move_reg(PPC_CTR, PPC_GPR(rS)); return flowContinue;
 		}
+	case 8:
+		if (spr1 == 0) {
+			move_reg(PPC_VRSAVE, PPC_GPR(rS)); 
+			return flowContinue;
+		}
 	}
 	ppc_opc_gen_check_privilege();
 	switch (spr2) {
@@ -1479,7 +1613,6 @@ JITCFlow ppc_opc_gen_mtspr()
 			jitcClobberAll();
 			asmCALL((NativeAddress)ppc_mmu_set_sdr1_check_error);
 			asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-			asmALURegImm(X86_MOV, ESI, gJITC.pc);
 			asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 			return flowEndBlockUnreachable;
 		}
@@ -1493,6 +1626,16 @@ JITCFlow ppc_opc_gen_mtspr()
 		case 17: move_reg(PPC_SPRG(1), PPC_GPR(rS)); return flowContinue;
 		case 18: move_reg(PPC_SPRG(2), PPC_GPR(rS)); return flowContinue;
 		case 19: move_reg(PPC_SPRG(3), PPC_GPR(rS)); return flowContinue;
+		case 28:
+			jitcGetClientRegister(PPC_GPR(rS), NATIVE_REG | EAX);
+			jitcClobberAll();
+			asmCALL((NativeAddress)writeTBL);
+			return flowContinue;
+		case 29:
+			jitcGetClientRegister(PPC_GPR(rS), NATIVE_REG | EAX);
+			jitcClobberAll();
+			asmCALL((NativeAddress)writeTBU);
+			return flowContinue;
 		}
 		break;
 	case 16: {
@@ -1558,27 +1701,39 @@ JITCFlow ppc_opc_gen_mtspr()
 		jitcClobberAll();
 		asmCALL((NativeAddress)ppc_mmu_tlb_invalidate_all_asm);
 		asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-		asmALURegImm(X86_MOV, ESI, gJITC.pc);
 		asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 		return flowEndBlockUnreachable;
 	}
+	case 29:
+		switch (spr1) {
+		case 17: return flowContinue; //g4
+		case 24: return flowContinue; //g4
+		case 25: return flowContinue; //g4
+		case 26: return flowContinue; //g4
+		}
 	case 31:
 		switch (spr1) {
 		case 16: move_reg(PPC_HID0, PPC_GPR(rS)); return flowContinue;
+		case 17: return flowContinue; //g4
 		case 18: return flowContinue;
-		case 21: return flowContinue;
+		case 21: return flowContinue; //g4
+		case 22: return flowContinue;
+		case 23: return flowContinue;
 		case 27: return flowContinue;
 		case 28: return flowContinue;
 		case 29: return flowContinue;
 		case 30: return flowContinue;
+		case 31: return flowContinue; //g4
 		}
 	}
 	invalid:
+	jitcClobberAll();
 	asmMOVRegDMem(EAX, (uint32)&gCPU.current_code_base);
-	asmJMP((NativeAddress)blbl);
-	return flowContinue;
-	PPC_OPC_ERR("unknown spr %d:%d\n", spr1, spr2);
-	return flowEndBlockUnreachable;
+	asmALURegImm(X86_ADD, EAX, gJITC.pc);
+	asmALURegImm(X86_MOV, EDX, spr1);
+	asmALURegImm(X86_MOV, ECX, spr2);
+	asmCALL((NativeAddress)unknown_spr_warning);
+	return flowEndBlock;
 }
 /*
  *	mtsr		Move to Segment Register
@@ -1607,7 +1762,6 @@ JITCFlow ppc_opc_gen_mtsr()
 	asmCALL((NativeAddress)ppc_mmu_tlb_invalidate_all_asm);
 	// sync
 //	asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-//	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 //	asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 //	return flowEndBlockUnreachable;	
 	return flowContinue;
@@ -1648,7 +1802,6 @@ JITCFlow ppc_opc_gen_mtsrin()
 	asmCALL((NativeAddress)ppc_mmu_tlb_invalidate_all_asm);
 	// sync
 //	asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-//	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 //	asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 //	return flowEndBlockUnreachable;	
 	return flowContinue;
@@ -1677,7 +1830,6 @@ JITCFlow ppc_opc_gen_rfi()
 	asmCALL((NativeAddress)ppc_set_msr_asm);
 	byte modrm[6];
 	asmALURegMem(X86_MOV, EAX, modrm, x86_mem(modrm, REG_NO, (uint32)(&gCPU.srr[0])));
-	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	asmALURegImm(X86_AND, EAX, 0xfffffffc);
 	asmJMP((NativeAddress)ppc_new_pc_asm);
 	return flowEndBlockUnreachable;
@@ -1756,7 +1908,6 @@ JITCFlow ppc_opc_gen_tlbia()
 	ppc_opc_gen_check_privilege();
 	asmCALL((NativeAddress)ppc_mmu_tlb_invalidate_all_asm);
 	asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 	return flowEndBlockUnreachable;
 }
@@ -1786,7 +1937,6 @@ JITCFlow ppc_opc_gen_tlbie()
 	jitcClobberAll();
 	asmCALL((NativeAddress)ppc_mmu_tlb_invalidate_entry_asm);
 	asmALURegImm(X86_MOV, EAX, gJITC.pc+4);
-	asmALURegImm(X86_MOV, ESI, gJITC.pc);
 	asmJMP((NativeAddress)ppc_new_pc_rel_asm);
 	return flowEndBlockUnreachable;
 }
@@ -1924,3 +2074,16 @@ JITCFlow ppc_opc_gen_twi()
 		return flowContinue;
 	}
 }
+
+/*      dcba	    Data Cache Block Allocate
+ *      .???
+ */
+void ppc_opc_dcba()
+{
+	// FIXME: check addr
+}
+JITCFlow ppc_opc_gen_dcba()
+{
+	// FIXME: check addr
+	return flowContinue;
+}
