@@ -566,7 +566,7 @@ void cuda_write(uint32 addr, uint32 data, int size)
 				ack = true;
 			}
 		}
-		if ((gCUDA.state == cuda_reading) && ack && (gCUDA.rACR & SR_OUT) 
+		if (gCUDA.state == cuda_reading && ack && (gCUDA.rACR & SR_OUT) 
 		&& !(!(gCUDA.rB & TIP) && (data & TIP))) {
 			// don't ask...
 			gCUDA.data[gCUDA.pos] = gCUDA.rSR;
@@ -577,7 +577,7 @@ void cuda_write(uint32 addr, uint32 data, int size)
 				IO_CUDA_ERR("cuda overflow!\n");
 			}
 		}
-		if ((gCUDA.state == cuda_writing) && ack) {
+		if (gCUDA.state == cuda_writing && ack) {
 			if (gCUDA.left <= 1) {
 				data |= TREQ;
 			}
@@ -621,8 +621,10 @@ void cuda_write(uint32 addr, uint32 data, int size)
 				gCUDA.left = 0;
 				gCUDA.pos = 0;
 			}
+			sys_lock_semaphore(gCUDA.idle_sem);
 			gCUDA.state = cuda_idle;
 			sys_signal_semaphore(gCUDA.idle_sem);
+			sys_unlock_semaphore(gCUDA.idle_sem);
 			IO_CUDA_TRACE2("CUDA CHANGE STATE %d: to %d\n", __LINE__, gCUDA.state);
 		} else {
 			gCUDA.rB = data;
@@ -797,6 +799,7 @@ void cuda_read(uint32 addr, uint32 &data, int size)
 		break;
 	case IFR:
 		data = gCUDA.rIFR;
+#if 0		
 		if (gCUDA.state == cuda_idle) {
 			if (!gCUDA.left /*&& !(gCUDA.rIER & SR_INT)*/) {
 				if (cuda_interrupt()) {
@@ -809,6 +812,7 @@ void cuda_read(uint32 addr, uint32 &data, int size)
 //			ht_printf("state not idle bla !\n");
 //			data |= SR_INT;
 		}
+#endif
 		cuda_update_T1();
 		IO_CUDA_TRACE("%d IFR->(%02x/%02x)\n", gCUDA.state, gCUDA.rIFR, data);
 		break;
@@ -827,11 +831,6 @@ void cuda_read(uint32 addr, uint32 &data, int size)
 	sys_unlock_mutex(gCUDAMutex);
 }
 
-bool cuda_interrupt()
-{
-	return false;
-}
-
 static sys_semaphore	gCUDAEventSem;
 static Queue		gCUDAEvents(true);
 
@@ -840,8 +839,8 @@ static bool cudaEventHandler(const SystemEvent &ev)
 	sys_lock_semaphore(gCUDAEventSem);
 //	ht_printf("queue  %d\n", ev.key.pressed);
 	gCUDAEvents.enQueue(new SystemEventObject(ev));
-	sys_unlock_semaphore(gCUDAEventSem);
 	sys_signal_semaphore(gCUDAEventSem);
+	sys_unlock_semaphore(gCUDAEventSem);
 	return true;
 }
 
@@ -918,9 +917,9 @@ static bool tryProcessCudaEvent(const SystemEvent &ev)
 			IO_CUDA_TRACE2("cuda not idle (%d)!\n", gCUDA.state);
 		}
 		sys_unlock_mutex(gCUDAMutex);
-		sys_lock_mutex(gCUDA.idle_sem);
+		sys_lock_semaphore(gCUDA.idle_sem);
 		sys_wait_semaphore_bounded(gCUDA.idle_sem, 10);
-		sys_unlock_mutex(gCUDA.idle_sem);
+		sys_unlock_semaphore(gCUDA.idle_sem);
 	}
 	IO_CUDA_WARN("Event processing timed out. Event dropped.\n");
 	return false;
@@ -937,7 +936,7 @@ static void *cudaEventLoop(void *arg)
 	while (1) {
 //		IO_CUDA_WARN("waiting on semaphore\n");
 		sys_wait_semaphore(gCUDAEventSem);
-//		IO_CUDA_WARN("semaphore signalled\n");
+//		IO_CUDA_WARN("semaphore signaled\n");
 		SystemEventObject *seo;
 		while ((seo = (SystemEventObject*)gCUDAEvents.deQueue())) {
 			tryProcessCudaEvent(seo->mEv);
