@@ -35,6 +35,7 @@
 #include "ide.h"
 #include "ata.h"
 #include "cd.h"
+#include "system/syscdrom.h"
 
 #define IDE_ADDRESS_ISA_BASE	0x1f0
 #define IDE_ADDRESS_ISA_BASE2	0x354
@@ -178,6 +179,7 @@ enum {
 #define IDE_ATAPI_SENSE_UNIT_ATTENTION	6
 
 #define IDE_ATAPI_ASC_INV_FIELD_IN_CMD_PACKET		0x24
+#define IDE_ATAPI_ASC_CANNOT_READ_MEDIUM		0x30
 #define IDE_ATAPI_ASC_MEDIUM_NOT_PRESENT		0x3a
 #define IDE_ATAPI_ASC_SAVING_PARAMETERS_NOT_SUPPORTED	0x39
 #define IDE_ATAPI_ASC_LOGICAL_BLOCK_OOR			0x21
@@ -212,7 +214,7 @@ struct IDEDriveState {
 	
 	int sectorpos;
 	int drqpos;
-	uint8 sector[2352];
+	uint8 sector[IDE_MAX_BLOCK_SIZE];
 	int current_sector_size;
 	int current_command;
 	uint32 dma_lba_start;
@@ -345,7 +347,7 @@ public:
 		mConfig[0x3d] = 1;
 		mConfig[0x3e] = 0x02;	// min grand
 		mConfig[0x3f] = 0x04;	// max latency
-	};
+	}
 	
 /*******************************************************************************
  *	IDE - Controller Core
@@ -364,7 +366,7 @@ public:
 			+ head * gIDEState.config[gIDEState.drive].hd.spt
         		+ sec_no - 1;
 		}
-	};
+	}
 
 	void incAddress()
 	{
@@ -389,7 +391,7 @@ public:
 				}
 			}
 		}
-	};
+	}
 	
 	void raiseInterrupt(int bus)
 	{
@@ -398,13 +400,13 @@ public:
 			pic_raise_interrupt(mConfig[0x3c]);
 		}
 		mConfig[MRDMODE] |= MRDMODE_INTR_CH0 << bus;
-	};
+	}
 	
 	void cancelInterrupt(int bus)
 	{
 		pic_cancel_interrupt(IO_PIC_IRQ_IDE0);
 		mConfig[MRDMODE] &= ~(MRDMODE_INTR_CH0 << bus);
-	};
+	}
 	
 void drive_ident()
 {
@@ -567,14 +569,14 @@ void drive_ident()
 		gIDEState.state[gIDEState.drive].sector[i*2] = id[i];
 		gIDEState.state[gIDEState.drive].sector[i*2+1] = id[i]>>8;
 	}
-};
+}
 
 	void atapi_command_nop()
 	{
 		gIDEState.state[gIDEState.drive].intr_reason |= IDE_ATAPI_INTR_REASON_C_D|IDE_ATAPI_INTR_REASON_I_O;
 		gIDEState.state[gIDEState.drive].intr_reason &= ~IDE_ATAPI_INTR_REASON_REL;
 		gIDEState.state[gIDEState.drive].status = IDE_STATUS_RDY | IDE_STATUS_SKC;
-	};
+	}
 
 	void atapi_command_error(uint8 sense_key, uint8 asc)
 	{
@@ -587,7 +589,7 @@ void drive_ident()
 		gIDEState.config[gIDEState.drive].cdrom.sense.sense_key = sense_key;
 		gIDEState.config[gIDEState.drive].cdrom.sense.asc = asc;
 		gIDEState.config[gIDEState.drive].cdrom.sense.ascq = 0;
-	};
+	}
 
 	void atapi_start_send_command(uint8 command, int reqlen, int alloclen, int sectorpos=0, int sectorsize=2048)
 	{
@@ -611,7 +613,7 @@ void drive_ident()
 		gIDEState.config[gIDEState.drive].cdrom.atapi.command = command;
 		gIDEState.config[gIDEState.drive].cdrom.atapi.drq_bytes = gIDEState.state[gIDEState.drive].byte_count;
 		gIDEState.config[gIDEState.drive].cdrom.atapi.total_remain = MIN(reqlen, alloclen);
-	};
+	}
 
 	void atapi_start_mode_sense(const byte *src, int size)
 	{
@@ -705,7 +707,7 @@ void receive_atapi_packet()
 		sector[7] = 0x00;
 
 		memcpy(sector+8, "SPIRO   ", 8);
-		memcpy(sector+16, "MULTIMAX 3000   ", 16);
+		memcpy(sector+16, "MULTIMAX 4000   ", 16);
 		memcpy(sector+32, "0.1 ", 4);
 		
 		sector[58] = 0x03;
@@ -811,134 +813,12 @@ void receive_atapi_packet()
 			// first check for ATAPI-SFF 8020 style
 			uint8 format = sector[2] & 0xf;
 			if (!format) {
-				// then for MMC-2 style 
+				// then for MMC-2 style
 				format = sector[9] >> 6;
 			}
-			switch (format) {
-			case 0: {
-				// .415
-				// start_track == track
-				atapi_start_send_command(command, 12, len);	
-				sector[0] = 0;
-				sector[1] = 10;
-				sector[2] = 1; // first track
-				sector[3] = 1; // last track
-				sector[4] = 0x00; // res
-				sector[5] = 0x16; // .408 (Data track, copy allowed :) )
-				sector[6] = 1;    // track number
-				sector[7] = 0x00; // res
-				if (msf) {
-//					SINGLESTEP("");
-					MSF m;
-					CDROMDevice::MSFfromLBA(m, 0);
-					sector[8] = 0x00;
-					sector[9] = m.m;
-					sector[10] = m.s;
-					sector[11] = m.f; 
-				} else {
-					sector[8] = 0x00; // LBA
-					sector[9] = 0x00;
-					sector[10] = 0x00;
-					sector[11] = 0x00; // LBA 
-				}
-				break;				
-			}
-			case 1:
-				// .418 Multisession information
-				// start_track == 0
-				atapi_start_send_command(command, 12, len);
-				sector[0] = 0;
-				sector[1] = 10;
-				sector[2] = 1; // first session
-				sector[3] = 1; // last session
-				sector[4] = 0x00; // res
-				sector[5] = 0x16; // .408 (Data track, copy allowed :) )
-				sector[6] = 1;    // first track number in last complete session
-				sector[7] = 0x00; // res
-				if (msf) {
-					MSF m;
-					CDROMDevice::MSFfromLBA(m, 0);
-					sector[8] = 0x00;
-					sector[9] = m.m;
-					sector[10] = m.s;
-					sector[11] = m.f; 
-				} else {
-					sector[8] = 0x00; // LBA
-					sector[9] = 0x00;
-					sector[10] = 0x00;
-					sector[11] = 0x00; // LBA 
-				}
-				break;
-			case 2:
-				// .420 Raw TOC
-				// start_track == session number
-				atapi_start_send_command(command, 48, len);
-				sector[0] = 0;
-				sector[1] = 46;
-				sector[2] = 1; // first session
-				sector[3] = 1; // last session
-				// points a0-af tracks b0-bf				
-				sector[4] = 1;    // session number
-				sector[5] = 0x16; // .408 (Data track, copy allowed :) )
-				sector[6] = 0;    // track number
-				sector[7] = 0xa0; // point (lead-in)
-				sector[8] = 0x00; // min
-				sector[9] = 0x00; // sec
-				sector[10] = 0x00; // frame
-				sector[11] = 0x00;   // zero
-				sector[12] = 0x01;   // first track
-				sector[13] = 0x00;   // disk type
-				sector[14] = 0x00;   // 				
-
-				sector[15] = 1;    // session number
-				sector[16] = 0x16; // .408 (Data track, copy allowed :) )
-				sector[17] = 0;    // track number
-				sector[18] = 0xa1; // point
-				sector[19] = 0x00; // min
-				sector[20] = 0x00; // sec
-				sector[21] = 0x00; // frame
-				sector[22] = 0x00;   // zero
-				sector[23] = 0x01;   // last track
-				sector[24] = 0x00;   // 
-				sector[25] = 0x00;   // 
-
-				sector[26] = 1;    // session number
-				sector[27] = 0x16; // .408 (Data track, copy allowed :) )
-				sector[28] = 0;    // track number
-				sector[29] = 0xa2; // point (lead-out)
-				sector[30] = 0x00; // min
-				sector[31] = 0x00; // sec
-				sector[32] = 0x00; // frame
-				MSF msf;
-				// FIXME?
-				CDROMDevice::MSFfromLBA(msf, dev->getCapacity()+0);
-				sector[33] = 0x00;   // zero
-				sector[34] = msf.m;   // start
-				sector[35] = msf.s;   //  of
-				sector[36] = msf.f;   //  leadout
-
-				sector[37] = 1;    // session number
-				sector[38] = 0x16; // .408 (Data track, copy allowed :) )
-				sector[39] = 0;    // track number
-				sector[40] = 1;    // point
-				sector[41] = 0x00; // min
-				sector[42] = 0x00; // sec
-				sector[43] = 0x00; // frame
-				CDROMDevice::MSFfromLBA(msf, 0);
-				sector[44] = 0x00;   // zero
-				sector[45] = msf.m;  // start
-				sector[46] = msf.s;  //  of
-				sector[47] = msf.f;  //  track
-				break;
-			case 3:
-				// PMA
-			case 4:
-				// ATIP
-			case 5: 
-				// CDTEXT
-			default:
-				IO_IDE_ERR("read toc: format %d not supported.\n", format);
-			}
+			len = MIN(len, IDE_MAX_BLOCK_SIZE);
+			int result = dev->readTOC(sector, msf, start_track, len, format);
+			atapi_start_send_command(command, result, len);
 		} else {
 			atapi_command_error(IDE_ATAPI_SENSE_NOT_READY, IDE_ATAPI_ASC_MEDIUM_NOT_PRESENT);
 		}
@@ -1000,7 +880,7 @@ void receive_atapi_packet()
 				atapi_start_mode_sense(sector+8, 36);
 				sector[8] = 0x2a;
 				sector[9] = 28;
-				sector[10] = 0x3; // CD-RW + CD-R read
+				sector[10] = 0x3b; // DVD-RAM + DVD-R + DVD-ROM + CD-RW + CD-R read
 				sector[11] = 0;
 				sector[12] = 0;
 				sector[13] = 3<<5;
@@ -1096,128 +976,20 @@ void receive_atapi_packet()
 		int feature = ((uint16)sector[2]<<8)|(sector[3]);
 		IO_IDE_TRACE("get_config: RT=%x len=%d f=%x\n", RT, len, feature);
 		memset(sector, 0, sizeof gIDEState.state[gIDEState.drive].sector);
-		sector[0] = 0x00; // length msb
-		sector[1] = 0x00;
-		sector[2] = 0x00;
-		
-		sector[6] = 0x00;
-		sector[7] = 0x08; // Current: Profile 8: CDROM
-		if (RT == 2) {
-			switch (feature) {
-			case 0: // Profile List
-				sector[3] = 16-4;
-				sector[8] = 0x00;
-				sector[9] = 0x00;  // Profile List
-				sector[10] = 0x03; // Version 0, Persistent=1, Current=1
-				sector[11] = 0x04; // Additional Length
-				sector[12] = 0x00;
-				sector[13] = 0x08; // CDROM
-				sector[14] = 0x01; // active	
-				sector[15] = 0x00;
-				atapi_start_send_command(command, 16, len);
-				break;
-			case 0x21:  // Incremental Streaming Writable .192
-			case 0x2d:  // Track At Once Feature .212
-			case 0x2e:  // Session At Once Feature .214
-			case 0x103: // CD Audio External Play Feature .224				
-			case 0x106: // DVD CSS Feature .106
-				sector[3] = 4; // not av.
-				atapi_start_send_command(command, 8, len);
-				break;
-			default: 
-				sector[3] = 4; // not av.
-				atapi_start_send_command(command, 8, len);
-				IO_IDE_WARN("moep\n");
-			}
-			raiseInterrupt(0);
-			break;
-		}
-		if (RT != 0) IO_IDE_ERR("moepmoep\n");
-		sector[3] = 0x44; // length lsb (72-4)
-		
-		// Features:
-		//  Feature 1, Profile List
-		sector[8] = 0x00;
-		sector[9] = 0x00;  // Profile List
-		sector[10] = 0x03; // Version 0, Persistent=1, Current=1
-		sector[11] = 0x04; // Additional Length
-		sector[12] = 0x00;
-		sector[13] = 0x08; // CDROM
-		sector[14] = 0x01; // active	
-		sector[15] = 0x00;
-		
-		//  Feature 2, Core
-		sector[16] = 0x00;
-		sector[17] = 0x01; // Core
-		sector[18] = 0x03; // Version 0, Persistent=1, Current=1
-		sector[19] = 0x04; // Additional Length
-		sector[20] = 0x00;
-		sector[21] = 0x00;
-		sector[22] = 0x00;
-		sector[23] = 0x02; // 02=ATAPI
-
-		//  Feature 3, Morphing
-		sector[24] = 0x00;
-		sector[25] = 0x02; // Morphing
-		sector[26] = 0x07; // Version 1, Persistent=1, Current=1
-		sector[27] = 0x04; // Additional Length
-		sector[28] = 0x00; // ASYNC = 0 (ATAPI)
-		sector[29] = 0x00;
-		sector[30] = 0x00;
-		sector[31] = 0x00;
-		
-		//  Feature 4, Removable
-		sector[32] = 0x00;
-		sector[33] = 0x03; // Removable
-		sector[34] = 0x03; // Version 0, Persistent=1, Current=1
-		sector[35] = 0x04; // Additional Length
-		sector[36] = 0x19; // Tray-Type-Loading, Eject=1, Jmpr=0, LockAllow=1
-		sector[37] = 0x00;
-		sector[38] = 0x00;
-		sector[39] = 0x00;
-		
-		//  Feature 5, Random Readable
-		sector[40] = 0x00;
-		sector[41] = 0x10; // Random Readable
-		sector[42] = 0x03; // Version 0, Persistent=1, Current=1 [FIXME?]
-		sector[43] = 0x08; // Additional Length
-		sector[44] = 0x00; // Logical Block Size MSB
-		sector[45] = 0x00;
-		sector[46] = 0x08;
-		sector[47] = 0x00; // Logical Block Size LSB
-		sector[48] = 0x00; // Blocking MSB
-		sector[49] = 0x10; // Blocking LSB
-		sector[50] = 0x00;
-		sector[51] = 0x00; // PP=0
-		
-		//  Feature 6, CD Read
-		sector[52] = 0x00;
-		sector[53] = 0x1e; // CD Read
-		sector[54] = 0x0b; // Version 2, Persistent=1, Current=1 [FIXME?]
-		sector[55] = 0x04; // Additional Length
-		sector[56] = 0x00; // DAP=0, C2=0, CD-Text=0
-		sector[57] = 0x00;
-		sector[58] = 0x00;
-		sector[59] = 0x00;
-		
-		//  Feature 7, Power Managment
-		sector[60] = 0x01;
-		sector[61] = 0x00; // Power Managment
-		sector[62] = 0x03; // Version 0, Persistent=1, Current=1 [FIXME?]
-		sector[63] = 0x00; // Additional Length
-		
-		//  Feature 8, Timeout
-		sector[64] = 0x01;
-		sector[65] = 0x05; // Timeout
-		sector[66] = 0x07; // Version 0, Persistent=1, Current=1 [FIXME?]
-		sector[67] = 0x04; // Additional Length
-		sector[68] = 0x00; // Group 3=0
-		sector[69] = 0x00;
-		sector[70] = 0x00;
-		sector[71] = 0x00;
-		
-/*		IO_IDE_WARN("ATAPI command 0x%08x not impl.\n", command);
-		atapi_command_error(IDE_ATAPI_SENSE_ILLEGAL_REQUEST, IDE_ATAPI_ASC_INV_FIELD_IN_CMD_PACKET);*/
+		len = MIN(len, IDE_MAX_BLOCK_SIZE);
+		int size = dev->getConfig(sector, len, RT, feature);
+		atapi_start_send_command(command, size, len);
+		raiseInterrupt(0);
+		break;
+	}
+	case IDE_ATAPI_COMMAND_EVENT_INFO: {
+		bool polled = sector[1] & 1;
+		uint8 request = sector[4];
+		int len = ((uint16)sector[7]<<8)|(sector[8]);
+		uint8 control = sector[9];
+//		int size = dev->eventInfo(sector, len, polled, request, control);
+//		atapi_start_send_command(command, size, len);
+		atapi_command_error(IDE_ATAPI_SENSE_ILLEGAL_REQUEST, IDE_ATAPI_ASC_INV_FIELD_IN_CMD_PACKET);
 		raiseInterrupt(0);
 		break;
 	}
@@ -1279,6 +1051,27 @@ void receive_atapi_packet()
 		raiseInterrupt(0);
 		break;
 	}
+	case IDE_ATAPI_COMMAND_READ_DVD_S: {
+		uint8 subcommand = sector[1] & 0xf;
+		uint32 address = ((uint32)sector[2]<<24)|((uint32)sector[3]<<16)|((uint32)sector[4]<<8)|sector[5];
+		uint8 layer = sector[6];
+		uint8 format = sector[7];
+		int len = ((uint16)sector[8]<<8)|(sector[9]);
+		uint8 AGID = sector[10] >> 6;
+		uint8 control = sector[11];
+		len = MIN(len, IDE_MAX_BLOCK_SIZE);
+
+		int size = 0;
+
+		if (dev->isDVD()) {
+			size = dev->readDVDStructure(sector, len, subcommand, address, layer, format, AGID, control);
+			atapi_start_send_command(command, size, len);
+		} else {
+			atapi_command_error(IDE_ATAPI_SENSE_NOT_READY, IDE_ATAPI_ASC_CANNOT_READ_MEDIUM);
+		}
+		raiseInterrupt(0);
+		break;
+	}
 	case IDE_ATAPI_COMMAND_LOAD_CD:
 	case IDE_ATAPI_COMMAND_READ12:
 	case IDE_ATAPI_COMMAND_MECH_STATUS:
@@ -1286,13 +1079,11 @@ void receive_atapi_packet()
 	case IDE_ATAPI_COMMAND_READ_CD_MSF:
 	case IDE_ATAPI_COMMAND_SCAN:
 		IO_IDE_WARN("unknown ATAPI command 0x%08x\n", command);
-		IO_IDE_ERR("deshalb.\n");
 		atapi_command_error(IDE_ATAPI_SENSE_ILLEGAL_REQUEST, IDE_ATAPI_ASC_INV_FIELD_IN_CMD_PACKET);
 		raiseInterrupt(0);
 		break;
 	default:
 		IO_IDE_WARN("unknown ATAPI command 0x%08x\n", command);
-		IO_IDE_ERR("deshalb.\n");
 		atapi_command_error(IDE_ATAPI_SENSE_ILLEGAL_REQUEST, IDE_ATAPI_ASC_INV_FIELD_IN_CMD_PACKET);
 		raiseInterrupt(0);
 		break;
@@ -1303,35 +1094,23 @@ void receive_atapi_packet()
 	}
 }
 
-	bool bm_ide_dotransfer(bool &prd_exhausted, uint32 bmide_prd_addr, byte bmide_command, byte bmide_status, uint32 lba, uint32 count)
+	bool bm_ide_dotransfer(bool &prd_exhausted, uint32 prd_addr, byte bmide_command, byte bmide_status, uint32 lba, uint32 count)
 	{
 		IO_IDE_TRACE("BM IDE transfer: prd_addr = %08x, lba = %08x, size = %08x\n", bmide_prd_addr, lba, count ? count : gIDEState.state[gIDEState.drive].sector_count);
-//		printf("holla die waldfee v2\n");
 
 		struct prd_entry {
 			uint32 addr PACKED;
 			uint32 size PACKED;
 		};
 
-		uint32 prd_addr = ppc_word_from_BE(bmide_prd_addr);
-/*		byte *xx;
-		if (ppc_direct_physical_memory_handle(prd_addr, xx)) {
-			IO_IDE_ERR("false 47\n");
-			return false;
-		}*/
-//		prd_entry *prd = (prd_entry*)xx;
 		prd_entry prd;
-		ppc_dma_read(&prd, prd_addr, 8);
-
+		if (!ppc_dma_read(&prd, prd_addr, 8)) return false;
+		prd.addr = ppc_word_from_LE(prd.addr);
+		prd.size = ppc_word_from_LE(prd.size);
 		prd_exhausted = false;
-		int pr_left = ppc_word_from_LE(prd.size) & 0xffff;
+		int pr_left = prd.size & 0xffff;
 		if (!pr_left) pr_left = 64*1024;
-/*		byte *pr;
-		if (ppc_direct_physical_memory_handle(prd->addr, pr)) {
-			IO_IDE_ERR("false 46\n");
-			return false;
-		}*/
-		uint32 pr = ppc_word_from_LE(prd.addr);
+
 		bool write_to_mem = bmide_command & BM_IDE_CR_WRITE;
 		bool write_to_device = !write_to_mem;
 		while (true) {
@@ -1341,23 +1120,27 @@ void receive_atapi_packet()
 			do {
 				uint8 buffer[transfer_at_once];
 				if (write_to_device) {
-					ppc_dma_read(buffer, pr, transfer_at_once);
+					ppc_dma_read(buffer, prd.addr, transfer_at_once);
 					if (gIDEState.config[gIDEState.drive].device->write(buffer, transfer_at_once) != transfer_at_once) {
-						IO_IDE_ERR("false3\n");
+						gIDEState.config[gIDEState.drive].device->release();
+						IO_IDE_WARN("write failed!\n");
 						return false;
 					}
 				} else {
 					if (gIDEState.config[gIDEState.drive].device->read(buffer, transfer_at_once) != transfer_at_once) {
-						IO_IDE_ERR("false3\n");
+						gIDEState.config[gIDEState.drive].device->release();
+						IO_IDE_WARN("read failed!\n");
 						return false;
 					}
-					ppc_dma_write(pr, buffer, transfer_at_once);
+					ppc_dma_write(prd.addr, buffer, transfer_at_once);
 				}
 				pr_left -= transfer_at_once;
-				pr += transfer_at_once;
+				prd.addr += transfer_at_once;
 				to_transfer -= transfer_at_once;
 				if (pr_left < 0) {
-					IO_IDE_ERR("pr_left became negative!\n");
+					gIDEState.config[gIDEState.drive].device->release();
+					IO_IDE_WARN("pr_left became negative!\n");
+					return false;
 				}
 				if (!pr_left) {
         				if (prd.size & 0x80000000) {
@@ -1369,17 +1152,22 @@ void receive_atapi_packet()
 							ready = gIDEState.state[gIDEState.drive].sector_count > 1;
 						}
 						if (to_transfer || ready) {
-							IO_IDE_ERR("false1\n");
+							gIDEState.config[gIDEState.drive].device->release();
+							IO_IDE_WARN("no more prd's, but still something to transfer\n");
 							return false;
 						}
 						prd_exhausted = true;
 					} else {
 						// get next prd
 						prd_addr += 8;
-						ppc_dma_read(&prd, prd_addr, 8);
+						if (!ppc_dma_read(&prd, prd_addr, 8)) {
+							gIDEState.config[gIDEState.drive].device->release();
+							return false;
+						}
+						prd.addr = ppc_word_from_LE(prd.addr);
+						prd.size = ppc_word_from_LE(prd.size);
 						pr_left = prd.size & 0xffff;
 						if (!pr_left) pr_left = 64*1024;
-						pr = ppc_word_from_LE(prd.addr);
 						transfer_at_once = to_transfer;
 						if (transfer_at_once > pr_left) transfer_at_once = pr_left;
 					}
@@ -1395,7 +1183,7 @@ void receive_atapi_packet()
 		}
 		gIDEState.config[gIDEState.drive].device->release();
 		return true;
-        };
+        }
 
 	bool read_bmdma_reg(uint32 port, uint32 &data, uint size)
 	{
@@ -1430,8 +1218,9 @@ void receive_atapi_packet()
 			break;
 		case 4:
 			if (size==4) {
-				IO_IDE_TRACE("bmide prd address: %08x(endianess)\n", mConfig[DTPR0]);
-				memcpy(&data, &mConfig[DTPR0], 4);// FIXME: endianess
+				memcpy(&data, &mConfig[DTPR0], 4);
+				data = ppc_word_from_LE(data);
+				IO_IDE_TRACE("bmide prd address: %08x\n", data);
 				return true;
 			}
 			break;
@@ -1460,7 +1249,8 @@ void receive_atapi_packet()
 		gIDEState.state[gIDEState.drive].mode = IDE_TRANSFER_MODE_NONE;
 		bool prd_exhausted;
 		uint32 bmide_prd_addr;
-		memcpy(&bmide_prd_addr, &mConfig[DTPR0], 4);	// FIXME: endianess
+		memcpy(&bmide_prd_addr, &mConfig[DTPR0], 4);
+		bmide_prd_addr = ppc_word_from_LE(bmide_prd_addr);
 		if (bm_ide_dotransfer(prd_exhausted, bmide_prd_addr, 
 				mConfig[BMIDECR0], mConfig[BMIDESR0], 
 				gIDEState.state[gIDEState.drive].dma_lba_start, 
@@ -1549,7 +1339,8 @@ void receive_atapi_packet()
 		case 4:
 			if (size==4) {
 				IO_IDE_TRACE("bmide prd address: %08x\n", data);
-				memcpy(&mConfig[DTPR0], &data, 4);// FIXME: endianess
+				data = ppc_word_to_LE(data);
+				memcpy(&mConfig[DTPR0], &data, 4);
 				return true;
 			}
 		}
@@ -1568,8 +1359,8 @@ void receive_atapi_packet()
 //			IO_IDE_TRACE("data <- %04x\n", data);
 			switch (gIDEState.state[gIDEState.drive].current_command) {
 			case IDE_COMMAND_WRITE_SECTOR: 
-				gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++] = data >> 8;
-				gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++] = data;
+				*((uint16 *)&gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos]) = ppc_half_to_LE(data);
+				gIDEState.state[gIDEState.drive].sectorpos += 2;
 				if (gIDEState.state[gIDEState.drive].sectorpos == 512) {
 					if (gIDEState.state[gIDEState.drive].mode == IDE_TRANSFER_MODE_WRITE) {
 						uint32 pos = makeLogical(gIDEState.state[gIDEState.drive].head, 
@@ -1601,8 +1392,8 @@ void receive_atapi_packet()
 				if (gIDEState.state[gIDEState.drive].sectorpos >= IDE_ATAPI_PACKET_SIZE) {
 					IO_IDE_ERR("sectorpos >= PACKET_SIZE\n");
 				}
-				gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++] = data >> 8;
-				gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++] = data;
+				*((uint16 *)&gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos]) = ppc_half_to_LE(data);
+				gIDEState.state[gIDEState.drive].sectorpos += 2;
 				if (gIDEState.state[gIDEState.drive].sectorpos >= IDE_ATAPI_PACKET_SIZE) {
 					// ATAPI packet received
 					IDEDevice *dev = gIDEState.config[gIDEState.drive].device;
@@ -1938,8 +1729,8 @@ void receive_atapi_packet()
 			gIDEState.state[gIDEState.drive].cyl = ((data<<8)&0xff00) + (gIDEState.state[gIDEState.drive].cyl & 0xff);
 			return;
 		}
-		IO_IDE_ERR("hae?\n");
-	};
+		IO_IDE_ERR("write(%d) %08x to unknown IDE register %d\n", size, data, addr);
+	}
 
 void ide_read_reg(uint32 addr, uint32 &data, int size)
 {
@@ -1958,8 +1749,8 @@ void ide_read_reg(uint32 addr, uint32 &data, int size)
 		case IDE_COMMAND_READ_SECTOR: 
 		case IDE_COMMAND_IDENT:
 		case IDE_COMMAND_IDENT_ATAPI:
-			data = ((uint16)gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++])<<8;
-			data |= gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++];
+			data = ppc_half_from_LE(*((uint16 *)&gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos]));
+			gIDEState.state[gIDEState.drive].sectorpos += 2;
 //			IO_IDE_TRACE("data: %04x\n", data);
 			if (gIDEState.state[gIDEState.drive].sectorpos == 512) {
 				if (gIDEState.state[gIDEState.drive].mode == IDE_TRANSFER_MODE_READ) {
@@ -2027,8 +1818,8 @@ void ide_read_reg(uint32 addr, uint32 &data, int size)
 					IO_IDE_ERR("unknown atapi state\n");
 				}
 			}
-			data = ((uint16)gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++])<<8;
-			data |= gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos++];
+			data = ppc_half_from_LE(*((uint16 *)&gIDEState.state[gIDEState.drive].sector[gIDEState.state[gIDEState.drive].sectorpos]));
+			gIDEState.state[gIDEState.drive].sectorpos += 2;
 //			IO_IDE_TRACE("data: %04x\n", data);
 			gIDEState.state[gIDEState.drive].drqpos += 2;
 			if (gIDEState.state[gIDEState.drive].drqpos >= gIDEState.config[gIDEState.drive].cdrom.atapi.drq_bytes) {
@@ -2103,8 +1894,8 @@ void ide_read_reg(uint32 addr, uint32 &data, int size)
 		return;
 	}
 	}
-	IO_IDE_ERR("hae?\n");
-};
+	IO_IDE_ERR("write(%d) %08x to unknown IDE register %d\n", size, data, addr);
+}
 
 /********************************************************************************
  *	PCI Interface
@@ -2122,7 +1913,7 @@ void ide_read_reg(uint32 addr, uint32 &data, int size)
 			return read_bmdma_reg(port, data, size);
 		}
 		return false;
-	};
+	}
 	
 	virtual bool	writeDeviceIO(uint r, uint32 port, uint32 data, uint size)
 	{
@@ -2137,7 +1928,7 @@ void ide_read_reg(uint32 addr, uint32 &data, int size)
 			return write_bmdma_reg(port, data, size);
 		}
 		return false;
-	};
+	}
 	
 	virtual void	readConfig(uint reg)
 	{
@@ -2146,7 +1937,7 @@ void ide_read_reg(uint32 addr, uint32 &data, int size)
 			// hook here, if you need notify on read
 		}
 		PCI_Device::readConfig(reg);
-	};
+	}
 	
 	virtual void writeConfig(uint reg, int offset, int size)
 	{
@@ -2158,12 +1949,12 @@ void ide_read_reg(uint32 addr, uint32 &data, int size)
 		if (reg >= BMIDECR0 && reg <= DTPR0) {
 			// FIXME: please fix this. I won't.
 			if (size != 1) IO_IDE_ERR("size != 1 bla in writeConfig()\n");
-			uint32 data = gPCI_Data >> (offset*8);
+			uint32 data = (gPCI_Data >> (offset*8)) & 0xff;
 			write_bmdma_reg(reg-BMIDECR0+offset, data, size);
 			return ;
 		}
 		PCI_Device::writeConfig(reg, offset, size);
-	};
+	}
 	
 };
 
@@ -2203,7 +1994,7 @@ void ide_init()
 		const char *imgkey = imgkeys[DISK];
 		if (gConfig->getConfigInt(instkey)) {
 			const char *masterslave[] = {"master", "slave"};
-			if (!gConfig->haveKey(imgkey)) throw new MsgfException("no disk image specified for ide%d %s.", 0, masterslave[DISK]);
+			if (!gConfig->haveKey(imgkey)) throw MsgfException("no disk image specified for ide%d %s.", 0, masterslave[DISK]);
 			String img, tmp, ext;
 			gConfig->getConfigString(imgkey, img);
 			if (gConfig->haveKey(typekey)) {
@@ -2213,8 +2004,12 @@ void ide_init()
 					ext = "img";
 				} else if (type == (String)"cdrom") {
 					ext = "iso";
+				} else if (type == (String)"dvdrom") {
+					ext = "dvd";
+				} else if (type == (String)"nativecdrom") {
+					ext = "nativecdrom";
 				} else {
-					IO_IDE_ERR("key '%s' must be set to 'hd' or 'cdrom'\n", typekey);
+					IO_IDE_ERR("key '%s' must be set to 'hd', 'cdrom' or 'nativecdrom'\n", typekey);
 				}
 			} else {
 				// type isn't specified, so we must rely on the file extension
@@ -2224,9 +2019,9 @@ void ide_init()
 			}
 			String name;
 			name.assignFormat("ide%d", DISK);
-			if (ext == (String)"img") {
+			if (ext == "img") {
 				gIDEState.config[DISK].protocol = IDE_ATA;
-				gIDEState.config[DISK].device = new ATADeviceFile(name, img);
+				gIDEState.config[DISK].device = new ATADeviceFile(name.contentChar(), img.contentChar());
 				const char *error;
 				if ((error = gIDEState.config[DISK].device->getError())) IO_IDE_ERR("%s\n", error);
 				gIDEState.config[DISK].hd.cyl = ((ATADevice*)gIDEState.config[DISK].device)->mCyl;
@@ -2234,17 +2029,40 @@ void ide_init()
 				gIDEState.config[DISK].hd.spt = ((ATADevice*)gIDEState.config[DISK].device)->mSpt;
 				gIDEState.config[DISK].bps = 512;
 				gIDEState.config[DISK].lba = true;
-			} else if (ext == (String)"iso") {
+			} else if (ext == "iso") {
 				gIDEState.config[DISK].protocol = IDE_ATAPI;
-				gIDEState.config[DISK].device = new CDROMDeviceFile(name);
-				((CDROMDeviceFile *)gIDEState.config[DISK].device)->changeDataSource(img);
+				gIDEState.config[DISK].device = new CDROMDeviceFile(name.contentChar());
+				((CDROMDeviceFile *)gIDEState.config[DISK].device)->activateDVD(false);
+				((CDROMDeviceFile *)gIDEState.config[DISK].device)->changeDataSource(img.contentChar());
 				const char *error;
 				if ((error = gIDEState.config[DISK].device->getError())) IO_IDE_ERR("%s\n", error);
 				((CDROMDeviceFile *)gIDEState.config[DISK].device)->setReady(true);
 				gIDEState.config[DISK].bps = 2048;
 				gIDEState.config[DISK].lba = false;
+			} else if (ext == "dvd") {
+				gIDEState.config[DISK].protocol = IDE_ATAPI;
+				gIDEState.config[DISK].device = new CDROMDeviceFile(name.contentChar());
+				((CDROMDeviceFile *)gIDEState.config[DISK].device)->activateDVD(true);
+
+				((CDROMDeviceFile *)gIDEState.config[DISK].device)->changeDataSource(img.contentChar());
+				const char *error;
+				if ((error = gIDEState.config[DISK].device->getError())) IO_IDE_ERR("%s\n", error);
+				((CDROMDeviceFile *)gIDEState.config[DISK].device)->setReady(true);
+				gIDEState.config[DISK].bps = 2048;
+				gIDEState.config[DISK].lba = true;
+			} else if (ext == "nativecdrom") {
+				gIDEState.config[DISK].protocol = IDE_ATAPI;
+				CDROMDevice* cdrom = createNativeCDROMDevice(name.contentChar(), img.contentChar());
+				if (!cdrom)
+				    IO_IDE_ERR("Error creating native CDROM device\n");
+				gIDEState.config[DISK].device = cdrom;
+				const char *error;
+				if ((error = gIDEState.config[DISK].device->getError()))
+				    IO_IDE_ERR("%s\n", error);
+				gIDEState.config[DISK].bps = 2048;
+				gIDEState.config[DISK].lba = false;
 			} else {
-				IO_IDE_ERR("unknown disk image (file extension is neither 'img' nor 'iso').\n");
+				IO_IDE_ERR("unknown disk image (file extension is neither 'img' nor 'iso' nor native CD infor).\n");
 			}
 			gIDEState.config[DISK].installed = true;
 		} else {
