@@ -24,7 +24,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: btree.c,v 1.1.1.1 2004/05/05 22:46:14 seppel Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -43,18 +42,22 @@
 # include "record.h"
 # include "swab.h"
 
+#ifdef HAVE_MEMSET
+#    define bzero(_a_,_s_) memset (_a_,0,_s_)
+#endif
+
 /* Read the node from the given buffer and swap the bytes.
  *
  * return pointer after reading the structure
  */
-void* btree_readnode(btree_node_desc* node, void *p)
+char* btree_readnode(btree_node_desc* node, char *p)
 {
-    node->next	    = bswabU32_inc(p);
-    node->prev	    = bswabU32_inc(p);
-    node->kind	    = bswabU8_inc(p);
-    node->height    = bswabU8_inc(p);
-    node->num_rec   = bswabU16_inc(p);
-    node->reserved  = bswabU16_inc(p);
+    node->next	    = bswabU32_inc(&p);
+    node->prev	    = bswabU32_inc(&p);
+    node->kind	    = bswabU8_inc(&p);
+    node->height    = bswabU8_inc(&p);
+    node->num_rec   = bswabU16_inc(&p);
+    node->reserved  = bswabU16_inc(&p);
     return p;
 } 
 
@@ -62,14 +65,14 @@ void* btree_readnode(btree_node_desc* node, void *p)
  *
  * return pointer after writing the structure
  */
-void* btree_writenode(btree_node_desc* node, void *p)
+char* btree_writenode(btree_node_desc* node, char *p)
 {
-    bstoreU32_inc(p, node->next);
-    bstoreU32_inc(p, node->prev);
-    bstoreU8_inc (p, node->kind);
-    bstoreU8_inc (p, node->height);
-    bstoreU16_inc(p, node->num_rec);
-    bstoreU16_inc(p, node->reserved);
+    bstoreU32_inc(&p, node->next);
+    bstoreU32_inc(&p, node->prev);
+    bstoreU8_inc (&p, node->kind);
+    bstoreU8_inc (&p, node->height);
+    bstoreU16_inc(&p, node->num_rec);
+    bstoreU16_inc(&p, node->reserved);
     return p;
 } 
 
@@ -78,25 +81,25 @@ void* btree_writenode(btree_node_desc* node, void *p)
  *
  * return pointer after reading the structure
  */
-void* btree_readhead(btree_head* head, void *p)
+char* btree_readhead(btree_head* head, char *p)
 {
     int i;
-    head->depth		= bswabU16_inc(p);
-    head->root		= bswabU32_inc(p);
-    head->leaf_count    = bswabU32_inc(p);
-    head->leaf_head	= bswabU32_inc(p);
-    head->leaf_tail	= bswabU32_inc(p);
-    head->node_size	= bswabU16_inc(p);
-    head->max_key_len   = bswabU16_inc(p);
-    head->node_count    = bswabU32_inc(p);
-    head->free_nodes    = bswabU32_inc(p);
-    head->reserved1	= bswabU16_inc(p);
-    head->clump_size    = bswabU32_inc(p);
-    head->btree_type    = bswabU8_inc(p);
-    head->reserved2	= bswabU8_inc(p);
-    head->attributes    = bswabU32_inc(p);
+    head->depth		= bswabU16_inc(&p);
+    head->root		= bswabU32_inc(&p);
+    head->leaf_count    = bswabU32_inc(&p);
+    head->leaf_head	= bswabU32_inc(&p);
+    head->leaf_tail	= bswabU32_inc(&p);
+    head->node_size	= bswabU16_inc(&p);
+    head->max_key_len   = bswabU16_inc(&p);
+    head->node_count    = bswabU32_inc(&p);
+    head->free_nodes    = bswabU32_inc(&p);
+    head->reserved1	= bswabU16_inc(&p);
+    head->clump_size    = bswabU32_inc(&p);
+    head->btree_type    = bswabU8_inc(&p);
+    head->reserved2	= bswabU8_inc(&p);
+    head->attributes    = bswabU32_inc(&p);
     for (i=0; i < 16; i++)
-	head->reserved3[i] = bswabU32_inc(p);
+	head->reserved3[i] = bswabU32_inc(&p);
     return p;
 }
 
@@ -104,25 +107,25 @@ void* btree_readhead(btree_head* head, void *p)
  *
  * return pointer after reading the structure
  */
-void* btree_writehead(btree_head* head, void *p)
+char* btree_writehead(btree_head* head, char *p)
 {
     int i;
-    bstoreU16_inc(p, head->depth);
-    bstoreU32_inc(p, head->root);
-    bstoreU32_inc(p, head->leaf_count);
-    bstoreU32_inc(p, head->leaf_head);
-    bstoreU32_inc(p, head->leaf_tail);
-    bstoreU16_inc(p, head->node_size);
-    bstoreU16_inc(p, head->max_key_len);
-    bstoreU32_inc(p, head->node_count);
-    bstoreU32_inc(p, head->free_nodes);
-    bstoreU16_inc(p, head->reserved1);
-    bstoreU32_inc(p, head->clump_size);
-    bstoreU8_inc (p, head->btree_type);
-    bstoreU8_inc (p, head->reserved2);
-    bstoreU32_inc(p, head->attributes);
+    bstoreU16_inc(&p, head->depth);
+    bstoreU32_inc(&p, head->root);
+    bstoreU32_inc(&p, head->leaf_count);
+    bstoreU32_inc(&p, head->leaf_head);
+    bstoreU32_inc(&p, head->leaf_tail);
+    bstoreU16_inc(&p, head->node_size);
+    bstoreU16_inc(&p, head->max_key_len);
+    bstoreU32_inc(&p, head->node_count);
+    bstoreU32_inc(&p, head->free_nodes);
+    bstoreU16_inc(&p, head->reserved1);
+    bstoreU32_inc(&p, head->clump_size);
+    bstoreU8_inc (&p, head->btree_type);
+    bstoreU8_inc (&p, head->reserved2);
+    bstoreU32_inc(&p, head->attributes);
     for (i=0; i < 16; i++)
-	bstoreU32_inc(p, head->reserved3[i]);
+	bstoreU32_inc(&p, head->reserved3[i]);
     return p;
 }
 
@@ -361,7 +364,7 @@ node_buf* btree_node_by_index(btree* bt, UInt16 index, int flags)
 /** intialize the btree with the first entry in the fork */
 static int btree_init(btree* bt, volume* vol, hfsp_fork_raw* fork)
 { 
-    void		*p;
+    char		*p;
     char		buf[vol->blksize]; 
     UInt16		node_size;
     btree_node_desc*	node = &bt->head_node;
@@ -393,7 +396,7 @@ static int btree_init(btree* bt, volume* vol, hfsp_fork_raw* fork)
 	{
 	    p = volume_readfromfork(vol, nodebuf, fork, 0, bt->blkpernode,
 		 HFSP_EXTENT_DATA, bt->cnid);
-	    ((char*) p) += HEADER_RESERVEDOFFSET; // skip header
+	    p += HEADER_RESERVEDOFFSET; // skip header
 	}
 	
 	bt->alloc_bits = malloc(alloc_size);
@@ -402,8 +405,9 @@ static int btree_init(btree* bt, volume* vol, hfsp_fork_raw* fork)
 	memcpy(bt->alloc_bits, p, alloc_size);
     }
 
+    bt->attributes = 0;
     /*** for debugging ***/ 
-    // bt->attributes |= BTREE_HEADDIRTY;
+    // ;
 
     if (node_cache_init(&bt->cache, bt, bt->head.depth + EXTRA_CACHESIZE))
 	return -1;
