@@ -201,7 +201,7 @@ protected:
 
 void PCIReset()
 {
-	IO_RTL8139_TRACE ("PCIReset()\n");	// PCI config
+	IO_RTL8139_TRACE("PCIReset()\n");	// PCI config
 	memset(mConfig, 0, sizeof mConfig);
 	// 0-3 set by totalReset()
 //	mConfig[0x04] = 0x07;	// io+memory+master
@@ -227,7 +227,7 @@ void PCIReset()
 
 void totalReset()
 {
-	IO_RTL8139_TRACE ("totalReset()\n");	// PCI config
+	IO_RTL8139_TRACE("totalReset()\n");	// PCI config
 	/* FIXME: resetting can be done more fine-grained (see TotalReset cmd).
 	 *        this is reset ALL regs.
 	 */
@@ -311,10 +311,10 @@ void setCR(uint8 cr)
 
 void maybeRaiseIntr()
 {
-	IO_RTL8139_TRACE ("maybeRaiseIntr()\n");
+	IO_RTL8139_TRACE("maybeRaiseIntr()\n");
 	if (mRegisters.InterruptMask & mIntStatus) {
 		//mIntStatus |= IS_interruptLatch;
-		IO_RTL8139_TRACE ("Generating interrupt. mIntStatus=%04x\n", mIntStatus);
+		IO_RTL8139_TRACE("Generating interrupt. mIntStatus=%04x\n", mIntStatus);
 		pic_raise_interrupt(mConfig[0x3c]);
 	}
 }
@@ -327,7 +327,7 @@ void TxPacket(uint32 address, uint32 size)
 	uint32	psize;
 
 	p = pbuf;
-	IO_RTL8139_TRACE ("address: %08x, size: %04x\n", address, size);
+	IO_RTL8139_TRACE("address: %08x, size: %04x\n", address, size);
 	if (ppc_dma_read(pbuf, address, size)) {
 /*		if (mVerbose > 1) {
 			debugDumpMem(ppc_addr, size);
@@ -345,26 +345,15 @@ void TxPacket(uint32 address, uint32 size)
 			if (w == psize) {
 				IO_RTL8139_TRACE("EthTun: %d bytes sent.\n", psize);
 			} else {
-				IO_RTL8139_TRACE("EthTun: ARGH! send error: only %d of %d bytes sent\n", w, psize);
+				IO_RTL8139_WARN("EthTun: ARGH! send error: only %d of %d bytes sent\n", w, psize);
 			}
 		} else {
-			IO_RTL8139_TRACE("EthTun: ARGH! send error in packet driver.\n");
+			IO_RTL8139_WARN("EthTun: ARGH! send error in packet driver.\n");
 		}
 		maybeRaiseIntr();
 	}
 }
 
-inline uint32 swapData(uint32 data, uint size)
-{
-	switch (size) {
-		case 1: break;
-		case 2: data = ppc_half_to_BE(data); break;
-		case 4: data = ppc_word_to_BE(data); break;
-		default: IO_RTL8139_ERR("impossibile!\n");
-	}
-	return data;
-}
-
 public:
 rtl8139_NIC(EthTunDevice *aEthTun, const byte *mac)
 : PCI_Device("rtl8139 Network interface card", 0x1, 0xd)
@@ -433,7 +422,7 @@ void readConfig(uint reg)
 {
 	//if (mVerbose) IO_RTL8139_TRACE("readConfig %02x\n", reg);
 	if (reg >= 0xdc) {
-		IO_RTL8139_WARN("re\n");
+		IO_RTL8139_WARN("readConfig(%x)\n", reg);
 	}
 	sys_lock_mutex(mLock);
 	PCI_Device::readConfig(reg);
@@ -445,7 +434,7 @@ void writeConfig(uint reg, int offset, int size)
 	//if (mVerbose) IO_RTL8139_TRACE("writeConfig %02x, %d, %d\n", reg, offset, size);
 	sys_lock_mutex(mLock);
 	if (reg >= 0xdc) {
-		IO_RTL8139_WARN("jg\n");
+		IO_RTL8139_WARN("writeConfig(%x, %d, %d)\n", reg, offset, size);
 	}
 	PCI_Device::writeConfig(reg, offset, size);
 	sys_unlock_mutex(mLock);
@@ -465,14 +454,14 @@ bool readDeviceIO(uint r, uint32 port, uint32 &data, uint size)
 			IO_RTL8139_WARN("unaligned read from IntStatus\n");
 		}
 		IO_RTL8139_TRACE("read IntStatus = %04x\n", mIntStatus);
-		data = swapData(mIntStatus, 2);
+		data = mIntStatus;
 		mIntStatus = 0; // a read resets the interrupt status register
 		retval = true;
 	} else if ((port >= 0) && (port+size <= sizeof(Registers))) {
 		// read from (standard) register
 		data = 0;
 		memcpy(&data, ((byte*)&mRegisters)+port, size);
-		//data = swapData(data, size);
+
 		switch (port) {
 		case 0x48:
 			IO_RTL8139_TRACE("read Timer = %08x\n", data);
@@ -492,7 +481,6 @@ bool readDeviceIO(uint r, uint32 port, uint32 &data, uint size)
 			IO_RTL8139_TRACE("read reg %04x (size %d) = %08x\n", port, size, data);
 			break;
 		}
-		data = swapData(data, size);
 		retval = true;
 	}
 	sys_unlock_mutex(mLock);
@@ -510,7 +498,6 @@ bool writeDeviceIO(uint r, uint32 port, uint32 data, uint size)
 	sys_lock_mutex(mLock);
 //	IO_RTL8139_TRACE("writeDevice has mLock\n");
 	original = data;
-	data = swapData(data, size);
 	if (port == 0x37) {
 		// CommandReg (no matter which window)
 		if (size != 1) {
@@ -646,7 +633,7 @@ void handleRxQueue()
 	byte            rxPacket[MAX_PACKET_SIZE];
 	uint16          rxPacketSize;
 	byte		tmp;
-	byte		broadcast[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
+	static byte	broadcast[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
 
 	while (1) {
 		while (mEthTun->waitRecvPacket() != 0) {
