@@ -55,10 +55,6 @@ int autoCompare(const Object *a, const Object *b)
  *	Class Object
  */
 
-Object::Object(BuildCtorArg)
-{
-}
-
 Object::Object()
 {
 }
@@ -78,7 +74,7 @@ void Object::done()
 int Object::compareTo(const Object *obj) const
 {
 #ifdef HAVE_HT_OBJECTS
-	throw new NotImplementedException(HERE);
+	throw NotImplementedException(HERE);
 #else
 	return 1;
 #endif
@@ -112,7 +108,7 @@ int Object::toString(char *buf, int buflen) const
 Object *Object::clone() const
 {
 #ifdef HAVE_HT_OBJECTS
-	throw new NotImplementedException(HERE);
+	throw NotImplementedException(HERE);
 #else
 	return NULL;
 #endif
@@ -135,29 +131,17 @@ bool	Object::instanceOf(Object *o) const
 	return instanceOf(o->getObjectID());
 }
 
-void Object::load(ObjectStream &s)
-{
-}
-
 ObjectID Object::getObjectID() const
 {
 	return OBJID_OBJECT;
 }
 
-void	Object::store(ObjectStream &s) const
-{
-}
-
 #endif /* HAVE_HT_OBJECTS */
 
 /*
  *	Class Enumerator
  */
 
-Enumerator::Enumerator(BuildCtorArg a) : Object(a)
-{
-}
-
 Enumerator::Enumerator()
 {
 }
@@ -243,10 +227,6 @@ int Enumerator::toString(char *buf, int buflen) const
  *	Class Container
  */
 
-Container::Container(BuildCtorArg a) : Enumerator(a)
-{
-}
-
 Container::Container()
 {
 	hom_objid = OBJID_TEMP;
@@ -287,10 +267,6 @@ Object *Container::removeObj(const Object *sig)
 /*
  *	List
  */
-List::List(BuildCtorArg a) : Container(a)
-{
-}
-
 List::List()
 {
 }
@@ -311,10 +287,6 @@ List::List()
 
 #define ARRAY_ALLOC_GROW_ABSMAX	64*1024
 
-Array::Array(BuildCtorArg a) : List(a)
-{
-}
-
 Array::Array(bool oo, int prealloc)
 {
 	own_objects = oo;
@@ -346,7 +318,7 @@ void Array::delAll()
 	elems = NULL;
 }
 
-Object *Array::clone() const
+Array *Array::clone() const
 {
 	Array *a = new Array(own_objects, ecount);
 	for (uint i = 0; i<ecount; i++) {
@@ -366,39 +338,11 @@ bool Array::instanceOf(ObjectID id) const
 	return (id == getObjectID()) || List::instanceOf(id);
 }
 
-void Array::load(ObjectStream &s)
-{
-	own_objects = true;
-
-	GET_INT32D(s, ecount);
-	GET_INT32X(s, hom_objid);
-	acount = 0;
-	elems = NULL;
-	realloc(ecount);
-
-	for (uint i=0; i<ecount; i++) {
-		Object *obj;
-		s.getObject(obj, "element", hom_objid);
-		elems[i] = obj;
-	}
-}
-
 ObjectID Array::getObjectID() const
 {
 	return OBJID_ARRAY;
 }
 
-void Array::store(ObjectStream &s) const
-{
-	PUT_INT32D(s, ecount);
-	PUT_INT32X(s, hom_objid);
-
-	ASSERT((ecount == 0) || (hom_objid != OBJID_TEMP));
-	for (uint i = 0; i<ecount; i++) {
-		s.putObject(elems[i], "element", hom_objid);
-	}
-}
-
 #endif /* HAVE_HT_OBJECTS */
 
 int Array::calcNewBufferSize(int curbufsize, int min_newbufsize) const
@@ -488,7 +432,7 @@ Object *Array::get(ObjHandle h) const
 
 uint Array::getObjIdx(ObjHandle h) const
 {
-	return InvObjHandle ? InvIdx : handleToNative(h);
+	return h == InvObjHandle ? InvIdx : handleToNative(h);
 }
 
 ObjHandle Array::findByIdx(int i) const
@@ -620,10 +564,6 @@ void Array::insertAt(ObjHandle h, Object *obj)
  *	Class Stack
  */
 
-Stack::Stack(BuildCtorArg a) : Array(a)
-{
-}
-
 Stack::Stack(bool own_objects) : Array(own_objects)
 {
 }
@@ -653,10 +593,6 @@ ObjectID Stack::getObjectID() const
  *	Class LinkedList
  */
 
-LinkedList::LinkedList(BuildCtorArg a) : List(a)
-{
-}
-
 LinkedList::LinkedList(bool oo)
 {
 	own_objects = oo;
@@ -700,7 +636,7 @@ void LinkedList::freeObj(Object *obj) const
 	}
 }
 
-Object *LinkedList::clone() const
+LinkedList *LinkedList::clone() const
 {
 	LinkedList *l = new LinkedList(own_objects);
 	LinkedListNode *n = first, *m;
@@ -720,43 +656,11 @@ bool LinkedList::instanceOf(ObjectID id) const
 	return (id == getObjectID()) || List::instanceOf(id);
 }
 
-void LinkedList::load(ObjectStream &s)
-{
-	own_objects = true;
-	first = last = NULL;
-
-	ecount = 0;
-	int ecount;
-	GET_INT32D(s, ecount);
-	GET_INT32X(s, hom_objid);
-
-	for (int i=0; i<ecount; i++) {
-		Object *obj;
-		s.getObject(obj, "element", hom_objid);
-		insert(obj);
-	}
-}
-
 ObjectID LinkedList::getObjectID() const
 {
 	return OBJID_LINKED_LIST;
 }
 
-void LinkedList::store(ObjectStream &s) const
-{
-	PUT_INT32D(s, ecount);
-	PUT_INT32X(s, hom_objid);
-
-	ObjHandle h = findFirst();
-	ASSERT((h == InvObjHandle) || (hom_objid != OBJID_TEMP));
-	while (h != InvObjHandle) {
-		Object *o = get(h);
-
-		s.putObject(o, "element", hom_objid);
-		h = findNext(h);
-	}
-}
-
 #endif
 uint LinkedList::count() const
 {
@@ -771,7 +675,7 @@ int LinkedList::compareObjects(const Object *a, const Object *b) const
 void LinkedList::forceSetByIdx(int idx, Object *obj)
 {
 	// FIXME:
-	throw new NotImplementedException(HERE);
+	throw NotImplementedException(HERE);
 }
 
 Object *LinkedList::get(ObjHandle h) const
@@ -914,7 +818,7 @@ void LinkedList::insertAt(ObjHandle h, Object *obj)
 bool LinkedList::moveTo(ObjHandle from, ObjHandle to)
 {
 	// FIXME:
-	throw new NotImplementedException(HERE);
+	throw NotImplementedException(HERE);
 }
 
 bool LinkedList::set(ObjHandle h, Object *obj)
@@ -955,10 +859,6 @@ ObjHandle LinkedList::nativeToHandle(LinkedListNode *n) const
  *	Class Queue
  */
 
-Queue::Queue(BuildCtorArg a) : LinkedList(a)
-{
-}
-
 Queue::Queue(bool own_objects) : LinkedList(own_objects)
 {
 }
@@ -980,10 +880,6 @@ ObjectID Queue::getObjectID() const
  *	Class BinaryTree
  */
 
-BinaryTree::BinaryTree(BuildCtorArg a) : Container(a)
-{
-}
-
 BinaryTree::BinaryTree(bool oo, Comparator comp)
 {
 	root = NULL;
@@ -1141,7 +1037,7 @@ void BinaryTree::cloneR(BinTreeNode *node)
 	cloneR(node->right);
 }
 
-Object *BinaryTree::clone() const
+BinaryTree *BinaryTree::clone() const
 {
 	BinaryTree *c = new BinaryTree(own_objects, compare);
 	c->cloneR(root);
@@ -1154,63 +1050,11 @@ bool	BinaryTree::instanceOf(ObjectID id) const
 	return (id == getObjectID()) || Container::instanceOf(id);
 }
 
-void BinaryTree::loadR(ObjectStream &s, BinTreeNode **n, int l, int r)
-{
-	if (l > r) {
-		*n = NULL;
-		return;
-	}
-	*n = allocNode();
-	uint m = (l+r)/2;
-	loadR(s, &(*n)->left, l, m-1);
-
-	s.getObject((*n)->key, "element", hom_objid);
-
-	loadR(s, &(*n)->right, m+1, r);
-}
-
-void BinaryTree::load(ObjectStream &s)
-{
-	const void *m = getAtomValue(GETX_INT32(s, "comparator"));
-	if (!m) throw new MsgException("BinaryTree::load() : invalid 'comparator' !");
-	compare = (Comparator)m;
-
-	GET_INT32D(s, ecount);
-	GET_INT32X(s, hom_objid);
-	root = NULL;
-	own_objects = true;
-
-	if (ecount) loadR(s, &root, 0, ecount-1);
-}
-
 ObjectID BinaryTree::getObjectID() const
 {
 	return OBJID_BINARY_TREE;
 }
 
-void BinaryTree::storeR(ObjectStream &s, BinTreeNode *n) const
-{
-	if (!n) return;
-	storeR(s, n->left);
-
-	s.putObject(n->key, "element", hom_objid);
-
-	storeR(s, n->right);
-}
-
-void	BinaryTree::store(ObjectStream &s) const
-{
-	int aId = getAtomId((void*)compare);
-	if (!aId) throw new MsgException("BinaryTree::store() : comparator not registered !");
-	PUTX_INT32X(s, aId, "comparator");
-
-	PUT_INT32D(s, ecount);
-	PUT_INT32X(s, hom_objid);
-
-	ASSERT(hom_objid != OBJID_TEMP);
-	storeR(s, root);
-}
-
 #endif
 
 uint BinaryTree::count() const
@@ -1247,7 +1091,7 @@ Object *BinaryTree::get(ObjHandle h) const
 uint BinaryTree::getObjIdx(ObjHandle h) const
 {
 	// FIXME: implement it
-	throw new NotImplementedException(HERE);
+	throw NotImplementedException(HERE);
 }
 
 ObjHandle BinaryTree::findByIdx(int i) const
@@ -1391,10 +1235,6 @@ ObjHandle BinaryTree::nativeToHandle(BinTreeNode *n) const
  *	Class AVLTree
  */
 
-AVLTree::AVLTree(BuildCtorArg a) : BinaryTree(a)
-{
-}
-
 AVLTree::AVLTree(bool aOwnObjects, Comparator aComparator)
  : BinaryTree(aOwnObjects, aComparator)
 {
@@ -1468,7 +1308,7 @@ void AVLTree::cloneR(AVLTreeNode *node)
 	cloneR((AVLTreeNode *)node->right);
 }
 
-Object *AVLTree::clone() const
+AVLTree *AVLTree::clone() const
 {
 	AVLTree *c = new AVLTree(own_objects, compare);
 	c->cloneR((AVLTreeNode *)root);
@@ -1481,45 +1321,6 @@ bool AVLTree::instanceOf(ObjectID id) const
 	return (id == getObjectID()) || BinaryTree::instanceOf(id);
 }
 
-int AVLTree::loadR(ObjectStream &s, BinTreeNode *&n, int l, int r)
-{
-	if (l > r) {
-		n = NULL;
-		return 0;
-	}
-	n = allocNode();
-	uint m = (l+r)/2;
-
-	int L = loadR(s, n->left, l, m-1);
-
-	s.getObject(n->key, "element", hom_objid);
-
-	int R = loadR(s, n->right, m+1, r);
-
-	if (L < R) {
-		((AVLTreeNode *)n)->unbalance = +1;
-	} else if (L > R) {
-		((AVLTreeNode *)n)->unbalance = -1;
-	} else {
-		((AVLTreeNode *)n)->unbalance = 0;
-	}
-	return MAX(L, R)+1;
-}
-
-void AVLTree::load(ObjectStream &s)
-{
-	const void *m = getAtomValue(GETX_INT32X(s, "comparator"));
-	if (!m) throw new MsgException("AVLTree::load() : invalid 'comparator' !");
-	compare = (Comparator)m;
-
-	GET_INT32D(s, ecount);
-	GET_INT32X(s, hom_objid);
-	root = NULL;
-	own_objects = true;
-
-	loadR(s, root, 0, ecount-1);
-}
-
 ObjectID AVLTree::getObjectID() const
 {
 	return OBJID_AVL_TREE;
@@ -1900,10 +1701,6 @@ Object *AVLTree::remove(ObjHandle h)
  *   Class Set
  */
 
-Set::Set(BuildCtorArg a) : AVLTree(a)
-{
-}
-
 Set::Set(bool oo)
 : AVLTree(oo)
 {
@@ -1939,10 +1736,6 @@ void Set::unionWith(Set *b)
  *
  */
 
-KeyValue::KeyValue(BuildCtorArg a) : Object(a)
-{
-}
-
 KeyValue::KeyValue(Object *aKey, Object *aValue)
 {
 	mKey = aKey;
@@ -1957,7 +1750,7 @@ KeyValue::~KeyValue()
 	delete mValue;
 }
 
-Object *KeyValue::clone() const
+KeyValue *KeyValue::clone() const
 {
 	return new KeyValue(mKey->clone(), mValue ? mValue->clone() : NULL);
 }
@@ -1978,39 +1771,23 @@ bool KeyValue::instanceOf(ObjectID id) const
 	return id == getObjectID();
 }
 
-void KeyValue::load(ObjectStream &s)
-{
-	GET_OBJECT(s, mKey);
-	GET_OBJECT(s, mValue);
-}
-
 ObjectID KeyValue::getObjectID() const
 {
 	return OBJID_KEYVALUE;
 }
 
-void KeyValue::store(ObjectStream &s) const
-{
-	PUT_OBJECT(s, mKey);
-	PUT_OBJECT(s, mValue);
-}
-
 #endif
 
 /*
  *	Class SInt
  */
 
-SInt::SInt(BuildCtorArg a) : Object(a)
-{
-}
-
 SInt::SInt(signed int i)
 {
 	value = i;
 }
 
-Object *SInt::clone() const
+SInt *SInt::clone() const
 {
 	return new SInt(value);
 }
@@ -2033,36 +1810,23 @@ bool SInt::instanceOf(ObjectID id) const
 	return id == getObjectID();
 }
 
-void SInt::load(ObjectStream &s)
-{
-	GET_INT32D(s, value);
-}
-
 ObjectID SInt::getObjectID() const
 {
 	return OBJID_SINT;
 }
 
-void SInt::store(ObjectStream &s) const
-{
-	PUT_INT32D(s, value);
-}
-
 #endif
 
 /*
  *	A signed Integer (64-bit)
  */
-#ifdef HAVE_HT_OBJECTS
-SInt64::SInt64(BuildCtorArg) {}
-#endif
 
 SInt64::SInt64(sint64 i)
 {
 	value = i;
 }
 
-Object *SInt64::clone() const
+SInt64 *SInt64::clone() const
 {
 	return new SInt64(value);
 }
@@ -2091,36 +1855,23 @@ bool SInt64::instanceOf(ObjectID id) const
 	return id == getObjectID();
 }
 
-void SInt64::load(ObjectStream &s)
-{
-	GET_INT64D(s, value);
-}
-
 ObjectID SInt64::getObjectID() const
 {
 	return OBJID_SINT64;
 }
 
-void SInt64::store(ObjectStream &s) const
-{
-	PUT_INT64D(s, value);
-}
 #endif
 
 /*
  *	Class UInt
  */
 
-UInt::UInt(BuildCtorArg a) : Object(a)
-{
-}
-
 UInt::UInt(unsigned int i)
 {
 	value = i;
 }
 
-Object *UInt::clone() const
+UInt *UInt::clone() const
 {
 	return new UInt(value);
 }
@@ -2150,36 +1901,22 @@ bool UInt::instanceOf(ObjectID id) const
 	return id == getObjectID();
 }
 
-void UInt::load(ObjectStream &s)
-{
-	GET_INT32D(s, value);
-}
-
 ObjectID UInt::getObjectID() const
 {
 	return OBJID_UINT;
 }
 
-void UInt::store(ObjectStream &s) const
-{
-	PUT_INT32D(s, value);
-}
-
 #endif
 
 /*
  *	A unsigned Integer (64-bit)
  */
-#ifdef HAVE_HT_OBJECTS
-UInt64::UInt64(BuildCtorArg) {}
-#endif
-
 UInt64::UInt64(uint64 i)
 {
 	value = i;
 }
 
-Object *UInt64::clone() const
+UInt64 *UInt64::clone() const
 {
 	return new UInt64(value);
 }
@@ -2208,39 +1945,23 @@ bool UInt64::instanceOf(ObjectID id) const
 	return id == getObjectID();
 }
 
-void UInt64::load(ObjectStream &s)
-{
-	GET_INT64D(s, value);
-}
-
 ObjectID UInt64::getObjectID() const
 {
 	return OBJID_UINT64;
 }
 
-void UInt64::store(ObjectStream &s) const
-{
-	PUT_INT64D(s, value);
-}
-
 #endif
 
 /*
  *	A floating-point number	(FIXME: no portable storage yet)
  */
 
-#ifdef HAVE_HT_OBJECTS
-Float::Float(BuildCtorArg)
-{
-}
-#endif
-
 Float::Float(double d)
 {
 	value = d;
 }
 
-Object *Float::clone() const
+Float *Float::clone() const
 {
 	return new Float(value);
 }
@@ -2270,15 +1991,11 @@ bool Float::instanceOf(ObjectID id) const
 	return id == getObjectID();
 }
 
-//	virtual	void		load(ObjectStream &s);
-
 ObjectID Float::getObjectID() const
 {
 	return OBJID_FLOAT;
 }
 
-//	virtual	void		store(ObjectStream &s) const;
-
 #endif
 
 /*
@@ -2294,10 +2011,6 @@ Pointer::Pointer(void *p)
  *	A memory area.
  */
 
-MemArea::MemArea(BuildCtorArg)
-{
-}
-
 MemArea::MemArea(const void *p, uint s, bool d)
 {
 	duplicate = d;
@@ -2317,7 +2030,7 @@ MemArea::~MemArea()
 	if (duplicate) free(ptr);
 }
 
-Object *MemArea::clone() const
+MemArea *MemArea::clone() const
 {
 	return new MemArea(ptr, size, true);
 }
@@ -2332,7 +2045,7 @@ int MemArea::compareTo(const Object *obj) const
 
 int MemArea::toString(char *buf, int buflen) const
 {
-	throw new NotImplementedException(HERE);
+	throw NotImplementedException(HERE);
 }
 
 #ifdef HAVE_HT_OBJECTS
@@ -2341,25 +2054,11 @@ bool MemArea::instanceOf(ObjectID id) const
 	return (id == getObjectID()) || Object::instanceOf(id);
 }
 
-void MemArea::load(ObjectStream &s)
-{
-	GET_INT32D(s, size);
-	ptr = malloc(size);
-	if (!ptr) throw std::bad_alloc();
-	GET_BINARY(s, ptr, size);
-}
-
 ObjectID MemArea::getObjectID() const
 {
 	return OBJID_MEMAREA;
 }
 
-void MemArea::store(ObjectStream &s) const
-{
-	PUT_INT32D(s, size),
-	PUT_BINARY(s, ptr, size);
-}
-
 #endif
 
 /*
@@ -2396,76 +2095,11 @@ bool quickSort(List &l)
  *	Module Init/Done
  */
 
-BUILDER(OBJID_OBJECT, Object);
-
-BUILDER(OBJID_ARRAY, Array);
-BUILDER(OBJID_STACK, Stack);
-
-BUILDER(OBJID_BINARY_TREE, BinaryTree);
-BUILDER(OBJID_AVL_TREE, AVLTree);
-BUILDER(OBJID_SET, Set);
-
-BUILDER(OBJID_LINKED_LIST, LinkedList);
-BUILDER(OBJID_QUEUE, Queue);
-//BUILDER(OBJID_DBL_LINKED_LIST, DblLinkedList);
-
-BUILDER(OBJID_KEYVALUE, KeyValue);
-BUILDER(OBJID_SINT, SInt);
-BUILDER(OBJID_UINT, UInt);
-BUILDER(OBJID_MEMAREA, MemArea);
-
-BUILDER(OBJID_STRING, String);
-BUILDER(OBJID_ISTRING, IString);
-
 bool initData()
 {
-	registerAtom(OBJID_AUTO_COMPARE, (void*)&autoCompare);
-
-	REGISTER(OBJID_OBJECT, Object);
-
-	REGISTER(OBJID_ARRAY, Array);
-	REGISTER(OBJID_STACK, Stack);
-
-	REGISTER(OBJID_BINARY_TREE, BinaryTree);
-	REGISTER(OBJID_AVL_TREE, AVLTree);
-	REGISTER(OBJID_SET, Set);
-
-	REGISTER(OBJID_LINKED_LIST, LinkedList);
-	REGISTER(OBJID_QUEUE, Queue);
-//	REGISTER(OBJID_DBL_LINKED_LIST, DblLinkedList);
-
-	REGISTER(OBJID_KEYVALUE, KeyValue);
-	REGISTER(OBJID_SINT, SInt);
-	REGISTER(OBJID_UINT, UInt);
-	REGISTER(OBJID_MEMAREA, MemArea);
-
-	REGISTER(OBJID_STRING, String);
-	REGISTER(OBJID_ISTRING, IString);
 	return true;
 } 
  
 void doneData()
 {
-	unregisterAtom(OBJID_AUTO_COMPARE);
-
-	UNREGISTER(OBJID_OBJECT, Object);
-
-	UNREGISTER(OBJID_ARRAY, Array);
-	UNREGISTER(OBJID_STACK, Stack);
-
-	UNREGISTER(OBJID_BINARY_TREE, BinaryTree);
-	UNREGISTER(OBJID_AVL_TREE, AVLTree);
-	UNREGISTER(OBJID_SET, Set);
-
-	UNREGISTER(OBJID_LINKED_LIST, LinkedList);
-	UNREGISTER(OBJID_QUEUE, Queue);
-//	UNREGISTER(OBJID_DBL_LINKED_LIST, DblLinkedList);
-
-	UNREGISTER(OBJID_KEYVALUE, KeyValue);
-	UNREGISTER(OBJID_SINT, SInt);
-	UNREGISTER(OBJID_UINT, UInt);
-	UNREGISTER(OBJID_MEMAREA, MemArea);
-
-	UNREGISTER(OBJID_STRING, String);
-	UNREGISTER(OBJID_ISTRING, IString);
 }
